#!/usr/bin/awk -f
# nono : scan logs to seek and ban IP with inappropriate activity
# Authors: 
#	prx <prx@si3t.ch>
# 	Original idea : solene <solene@openbsd.org>
# licence: MIT
# usage:
#	tail -f /var/www/some_log | nono

function usage() {

	printf "tail -f /var/log/authlog /var/log/maillog ... | nono.awk\n"
	printf "    available options:\n"
	printf "        -v TRIES=5\n"
	printf "        -v BANCMD=\"pfctl -T add -t bot\"\n"
	printf "        -v KILLSTATE=\"pfctl -k\"\n"
	printf "        -v IGNORE=/etc/nono.ignore\n"

	exit
}

function isignored(ip) {
	ret = 0
	while ((getline current_ip < IGNORE) == 1 ) {
		if (ip == current_ip ) {
			ret = 1
			break
		}
	}
	close(IGNORE)

	return ret
}

function ban(seen, ip, reason) {
	seen[ip]++
	if(seen[ip] >= tries) {
		if (isignored(ip) == 0) {
			system(sprintf("logger -t \"nono\" \"%s is naughty and is now banned because: %s\"", ip, reason))
			system(sprintf("%s  %s", BANCMD, ip))
			system(sprintf("%s  %s", KILLSTATE, ip))
		}
		seen[ip] = 0 # reset
	}
}

BEGIN {

	if ((ARGV[2] == "-h") || (ARGV[2] == "--help"))  {
		usage()
	}

	# check for options or set defaults
	if ( ! TRIES ) { TRIES = 5 }
	if ( ! BANCMD ) { BANCMD = "pfctl -T add -t bot" }
	if ( ! KILLSTATE ) { KILLSTATE = "pfctl -k" }
	if ( ! IGNORE ) { IGNORE = "/etc/nono.ignore" }
}

## sshd

/ sshd.* Failed password | sshd.* Invalid user / {
	match($0, "from ([0-9\.:abcdef]*)")
	if ( RLENGTH != -1 ) {
		shift = 5 # length("from ") = 5
		ip = substr($0, RSTART + shift , RLENGTH - shift)
		ban(seen, ip, "ssh wrong pw or invalid user")
	}
	next
}
/ sshd.*Disconnected from authenticating user root / {
	ip = $11
	ban(seen, ip, "ssh root attempt")
	next
}


## smtpd

/ smtp connected / {
	ip = substr($9, 9)
	sub("^\\[", "", ip)
	sub("\\]$", "", ip)
	ips[$6]=ip
}
/ smtp authentication / && /result=permfail/ {
	seen[ips[$6]]++
	ban(seen, ips[$6], "smtp auth fail")
	next
}

## dovecot

/ dovecot.* auth failed.* rip= / {
	ip = substr($19, 4)
	sub(",$", "", ip)
	ban(seen, ip, "dovecot failed auth")
	next
}

## httpd 
/ 401 / {
	if ( $9 == "401" ) {
		ip = $2
		ban(seen, ip, "401")
	}
	next
}
